@@ -1,26 +1,7 @@
 # -*- coding: utf-8 -*-
-
-#
-# Copyright © 2012-2013 by its contributors. See AUTHORS for details.
-#
+# Copyright © 2012-2014 by its contributors. See AUTHORS for details.
 # Distributed under the MIT/X11 software license, see the accompanying
 # file LICENSE or http://www.opensource.org/licenses/mit-license.php.
-#
-
-import calendar
-import numbers
-from struct import pack, unpack
-
-from recordtype import recordtype
-
-from .mixins import HashableMixin, SerializableMixin
-from .numeric import mpq
-from .script import Script
-from .serialize import (
-    serialize_varchar, deserialize_varchar,
-    serialize_hash, deserialize_hash,
-    serialize_list, deserialize_list)
-from .tools import StringIO, icmp, list, target_from_compact, tuple
 
 __all__ = [
     'ChainParameters',
@@ -31,6 +12,13 @@ __all__ = [
     'ConnectedBlockInfo',
 ]
 
+import calendar
+import numbers
+from struct import pack, unpack
+from recordtype import recordtype
+
+from .mixins import HashableMixin, SerializableMixin
+
 # ===----------------------------------------------------------------------===
 
 ChainParameters = recordtype('ChainParameters',
@@ -54,14 +42,18 @@ class Output(SerializableMixin):
         return getattr(cls, 'script_class', Script)
 
     def serialize(self):
-        result  = pack('<Q', self.amount)
-        result += self.contract.serialize()
-        return result
+        parts = list()
+        parts.append(pack('<Q', self.amount))
+        script = self.contract.serialize()
+        parts.append(LittleCompactSize(len(script)).serialize())
+        parts.append(FlatData(script).serialize())
+        return b''.join(parts)
     @classmethod
     def deserialize(cls, file_):
         initargs = {}
         initargs['amount'] = unpack('<Q', file_.read(8))[0]
-        initargs['contract'] = cls.get_script_class().deserialize(file_)
+        script_len = LittleCompactSize.deserialize(file_)
+        initargs['contract'] = cls.get_script_class().deserialize(file_, script_len)
         return cls(**initargs)
 
     def __eq__(self, other):
@@ -92,27 +84,28 @@ class Input(SerializableMixin):
         return getattr(cls, 'script_class', Script)
 
     def serialize(self):
-        result  = serialize_hash(self.hash, 32)
-        result += pack('<I', self.index)
-        if hasattr(self.endorsement, 'serialize'):
-            result += self.endorsement.serialize()
-        else:
-            result += serialize_varchar(self.endorsement) # <-- coinbase
-        result += pack('<I', self.sequence)
-        return result
+        parts = list()
+        parts.append(hash256.serialize(self.hash))
+        parts.append(pack('<I', self.index))
+        script = self.endorsement
+        if hasattr(script, 'serialize'):
+            script = script.serialize()
+        parts.append(LittleCompactSize(len(script)).serialize())
+        parts.append(FlatData(script).serialize())
+        parts.append(pack('<I', self.sequence))
+        return b''.join(parts)
     @classmethod
     def deserialize(cls, file_):
         initargs = {}
-        initargs['hash'] = deserialize_hash(file_, 32)
+        initargs['hash'] = hash256.deserialize(file_)
         initargs['index'] = unpack('<I', file_.read(4))[0]
-        str_ = deserialize_varchar(file_) # <-- might be coinbase!
-        initargs['sequence'] = unpack('<I', file_.read(4))[0]
+        len_ = LittleCompactSize.deserialize(file_)
         if not all((initargs['hash']  == 0,
                     initargs['index'] == 0xffffffff)):
-            initargs['coinbase'] = str_
+            initargs['coinbase'] = FlatData.deserialize(file_, len_)
         else:
-            initargs['endorsement'] = cls.get_script_class().deserialize(
-                StringIO(serialize_varchar(str_)))
+            initargs['endorsement'] = cls.get_script_class().deserialize(file_, len_)
+        initargs['sequence'] = unpack('<I', file_.read(4))[0]
         return cls(**initargs)
 
     @property
@@ -149,9 +142,9 @@ class Transaction(SerializableMixin, HashableMixin):
         if outputs is None: outputs = ()
         super(Transaction, self).__init__(*args, **kwargs)
         self.version = version
-        getattr(self, 'inputs_create', lambda:setattr(x, 'inputs', list()))
+        getattr(self, 'inputs_create', lambda:setattr(self, 'inputs', list()))()
         self.inputs.extend(inputs)
-        getattr(self, 'outputs_create', lambda:setattr(x, 'outputs', list()))
+        getattr(self, 'outputs_create', lambda:setattr(self, 'outputs', list()))()
         self.outputs.extend(outputs)
         self.lock_time = lock_time
         self.reference_height = reference_height
@@ -168,8 +161,8 @@ class Transaction(SerializableMixin, HashableMixin):
         if self.version not in (1,2):
             raise NotImplementedError()
         result  = pack('<I', self.version)
-        result += serialize_list(self.inputs, lambda i:i.serialize())
-        result += serialize_list(self.outputs, lambda o:o.serialize())
+        result += serialize_iterator(self.inputs, lambda i:i.serialize())
+        result += serialize_iterator(self.outputs, lambda o:o.serialize())
         result += pack('<I', self.lock_time)
         if self.version==2:
             result += pack('<I', self.reference_height)
@@ -186,8 +179,8 @@ class Transaction(SerializableMixin, HashableMixin):
         initargs['version'] = unpack('<I', file_.read(4))[0]
         if initargs['version'] not in (1,2):
             raise NotImplementedError()
-        initargs['inputs'] = list(deserialize_list(file_, lambda f:cls.deserialize_input(f)))
-        initargs['outputs'] = list(deserialize_list(file_, lambda f:cls.deserialize_output(f)))
+        initargs['inputs'] = list(deserialize_iterator(file_, lambda f:cls.deserialize_input(f)))
+        initargs['outputs'] = list(deserialize_iterator(file_, lambda f:cls.deserialize_output(f)))
         initargs['lock_time'] = unpack('<I', file_.read(4))[0]
         if initargs['version'] in (2,):
             initargs['reference_height'] = unpack('<I', file_.read(4))[0]
@@ -254,8 +247,8 @@ class Block(SerializableMixin, HashableMixin):
         if self.version not in (1,2):
             raise NotImplementedError()
         result  = pack('<I', self.version)
-        result += serialize_hash(self.parent_hash, 32)
-        result += serialize_hash(self.merkle_hash, 32)
+        result += hash256.serialize(self.parent_hash)
+        result += hash256.serialize(self.merkle_hash)
         if isinstance(self.time, numbers.Integral):
             time = self.time
         else:
@@ -272,8 +265,8 @@ class Block(SerializableMixin, HashableMixin):
         initargs['version'] = unpack('<I', file_.read(4))[0]
         if initargs['version'] not in (1,2):
             raise NotImplementedError()
-        initargs['parent_hash'] = deserialize_hash(file_, 32)
-        initargs['merkle_hash'] = deserialize_hash(file_, 32)
+        initargs['parent_hash'] = hash256.deserialize(file_)
+        initargs['merkle_hash'] = hash256.deserialize(file_)
         initargs['time'] = unpack('<I', file_.read(4))[0]
         initargs['bits'] = unpack('<I', file_.read(4))[0]
         initargs['nonce'] = unpack('<I', file_.read(4))[0]
@@ -313,9 +306,15 @@ class Block(SerializableMixin, HashableMixin):
             return 0
         return (1<<256) // (target+1)
 
+# ===----------------------------------------------------------------------===
+
 ConnectedBlockInfo = recordtype('ConnectedBlockInfo',
     ['parent', 'height', 'aggregate_work'])
 
-#
-# End of File
-#
+# ===----------------------------------------------------------------------===
+
+from .hash import hash256
+from .numeric import mpq
+from .script import Script
+from .serialize import LittleCompactSize, FlatData, serialize_iterator, deserialize_iterator
+from .tools import StringIO, icmp, list, target_from_compact, tuple
