@@ -1,20 +1,14 @@
 # -*- coding: utf-8 -*-
-
-#
-# Copyright © 2012-2013 by its contributors. See AUTHORS for details.
-#
+# Copyright © 2012-2014 by its contributors. See AUTHORS for details.
 # Distributed under the MIT/X11 software license, see the accompanying
 # file LICENSE or http://www.opensource.org/licenses/mit-license.php.
-#
 
 import six
 
 from struct import pack, unpack
 
 from .mixins import SerializableMixin
-from .serialize import (
-    serialize_varchar, deserialize_varchar,
-    serialize_hash, deserialize_hash)
+from .serialize import LittleCompactSize, FlatData
 from .tools import StringIO
 
 # ===----------------------------------------------------------------------===
@@ -483,7 +477,7 @@ class ScriptOp(SerializableMixin, six.binary_type):
             return None
 
     def serialize(self):
-        return self
+        return b''.join((self,))
     @classmethod
     def deserialize(cls, file_):
         opcode = file_.read(1)
@@ -543,10 +537,7 @@ class ScriptOp(SerializableMixin, six.binary_type):
             return self.opcode - OP_1 + 1
         elif self.opcode in xrange(1,OP_PUSHDATA4+1):
             data = self.data[:-1] + chr(ord(self.data[-1])&0x7f)
-            bignum = deserialize_hash(StringIO(data), len(data))
-            if ord(self.data[-1]) & 0x80:
-                return -bignum
-            return bignum
+            return BigNum.deserialize(StringIO(data), len(data))
         else:
             raise ValueError(u"non-data script-op cannot be interpreted as integer")
 
@@ -560,10 +551,7 @@ class ScriptOp(SerializableMixin, six.binary_type):
         elif 1 <= value <= 16:
             self.opcode = OP_1 + value - 1
         else:
-            neg = value < 1
-            absv = abs(value)
-            data = serialize_hash(absv, 1+len(bin(long(absv)).rstrip('L')[2:])//8)
-            if neg: data = data[:-1] + chr(ord(data[-1])|0x80)
+            data = BigNum(value).serialize()
             datalen = len(data)
             if datalen < OP_PUSHDATA1:
                 self.opcode = datalen
@@ -605,10 +593,13 @@ class Script(SerializableMixin, six.binary_type):
             yield script_op_class.deserialize(file_)
 
     def serialize(self):
-        return serialize_varchar(self)
+        return b''.join((self,))
     @classmethod
-    def deserialize(cls, file_):
-        return cls(deserialize_varchar(file_))
+    def deserialize(cls, file_, len_):
+        return cls(FlatData.deserialize(file_, len_))
+
+    def join(self, *args, **kwargs):
+        return self.__class__(super(Script, self).join(*args, **kwargs))
 
     def __repr__(self):
         try:
@@ -619,7 +610,7 @@ class Script(SerializableMixin, six.binary_type):
 # ===----------------------------------------------------------------------===
 
 from .defaults import CLIENT_VERSION
-from .serialize import SER_DISK, serialize_varint, deserialize_varchar
+from .serialize import SER_DISK
 
 class ScriptPickler(object):
     """Compact serializer for scripts.
@@ -644,7 +635,6 @@ class ScriptPickler(object):
     def _dump(script, file_, protocol, version):
         if hasattr(script, 'serialize'):
             script = script.serialize()
-            script = deserialize_varchar(StringIO(script))
         script_len = len(script)
         if 23 == script_len and all(
             script[k:k+1] == six.int2byte(v)
@@ -676,7 +666,7 @@ class ScriptPickler(object):
                         66: OP_CHECKSIG})):
             str_ = b''.join([six.int2byte(0x04 | ord(script[65:66])&0x01), script[2:34]])
         else:
-            str_ = b''.join([serialize_varint(script_len + 0x06), script])
+            str_ = b''.join([LittleCompactSize(script_len + 0x06).serialize(), script])
         file_.write(str_)
 
     @staticmethod
@@ -685,7 +675,6 @@ class ScriptPickler(object):
         if size == 0:
             hash_ = file_.read(20)
             return b''.join([
-                six.int2byte(25),
                 six.int2byte(OP_DUP),
                 six.int2byte(OP_HASH160),
                 six.int2byte(20), hash_,
@@ -695,7 +684,6 @@ class ScriptPickler(object):
         elif size == 1:
             hash_ = file_.read(20)
             return b''.join([
-                six.int2byte(23),
                 six.int2byte(OP_HASH160),
                 six.int2byte(20), hash_,
                 six.int2byte(OP_EQUAL),
@@ -703,7 +691,6 @@ class ScriptPickler(object):
         elif size in (2, 3):
             compressed_key = file_.read(32)
             return b''.join([
-                six.int2byte(35),
                 six.int2byte(33),
                 six.int2byte(size),
                 compressed_key,
@@ -714,7 +701,6 @@ class ScriptPickler(object):
                 b''.join([six.int2byte(size-2), file_.read(32)])))
             verifying_key.compressed = False
             return b''.join([
-                six.int2byte(67),
                 six.int2byte(65),
                 verifying_key.serialize(),
                 six.int2byte(OP_CHECKSIG),
@@ -726,8 +712,7 @@ class ScriptPickler(object):
         elif size == 0xff:
             size = unpack("<Q", file_.read(8))[0]
         size = size - 6
-        script = file_.read(size)
-        return serialize_varchar(script)
+        return FlatData.deserialize(file_, size)
 
     def get_script_class(self):
         return getattr(self, 'script_class', Script)
@@ -750,13 +735,13 @@ class ScriptPickler(object):
             file = self._file
         script_class = self.get_script_class()
         script = self._load(file, self._protocol, self._version)
-        return script_class.deserialize(StringIO(script))
+        return script_class(script)
 
     def loads(self, string):
         "Decompress the passed-in compact script and return the result."
         script_class = self.get_script_class()
         script = self._load(StringIO(string), self._protocol, self._version)
-        return script_class.deserialize(StringIO(script))
+        return script_class(script)
 
 ScriptUnpickler = ScriptPickler
 
@@ -773,7 +758,3 @@ __all__ = OPCODE_NAMES.values() + [
 ]
 
 from .crypto import VerifyingKey
-
-#
-# End of File
-#
